@@ -223,7 +223,7 @@ audio_properties_view_new ()
 			     "/nautilus/glade/audio-properties-view.glade",
 			     "content", NULL);
 
-	g_return_if_fail (xml != NULL);
+	g_return_val_if_fail (xml != NULL, NULL);
 
 	/* FIXME: check results */
 	view->vbox = glade_xml_get_widget (xml, "content");
