@@ -147,6 +147,19 @@ audio_view_tree_model_dump (AudioView *view)
 #endif
 
 /* helper functions */
+static void
+show_error_dialog (AudioView *view, const gchar *message)
+{
+  GtkWidget *parent, *dialog;
+
+  parent = gtk_widget_get_toplevel (view->widget);
+  dialog = gtk_message_dialog_new (GTK_WINDOW (parent), GTK_DIALOG_MODAL,
+					GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+					"%s", message);
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+}
+
 static GtkWidget *
 control_box_add_labeled_button (GtkWidget *control,
 		                const gchar *label, const gchar *file,
@@ -199,6 +212,7 @@ audio_view_is_playable (const char *uri)
         /* we get a newly allocated type with this function */
         /* FIXME: why do we need this cast ? it ought to return char * ! */
         type = (char *) gnome_vfs_get_mime_type (uri);
+	NM_DEBUG("audio_view_is_playable: gnome vfs type %s\n", type);
         if (!type) return FALSE;
         /* We also check for "old" mime types to catch everything */
         if (
@@ -270,10 +284,14 @@ audio_view_mime_to_type (const gchar *mime)
         if ((strcmp (mime, "application/x-ogg") == 0) ||
             (strcmp (mime, "application/ogg") == 0))
 		return g_strdup (_("Ogg/Vorbis"));
-        if (strcmp (mime, "application/x-flac") == 0)
+        if ((strcmp (mime, "application/x-flac") == 0) ||
+            (strcmp (mime, "audio/x-flac") == 0))
 		return g_strdup (_("FLAC"));
-        if ((strcmp (mime, "audio/x-mp3") == 0) ||
-            (strcmp (mime, "audio/mpeg") == 0))
+        else if ((strcmp (mime, "audio/mpeg") == 0) ||
+                 (strcmp (mime, "audio/x-mp3") == 0) ||
+                 (strcmp (mime, "audio/mp3") == 0) ||
+                 (strcmp (mime, "application/x-id3") == 0) ||
+                 (strcmp (mime, "audio/x-id3") == 0))
 		return g_strdup (_("MPEG"));
         if (strcmp (mime, "audio/x-wav") == 0)
 		return g_strdup (_("WAVE"));
@@ -296,37 +314,82 @@ audio_view_mime_to_type (const gchar *mime)
 	return g_strdup (_("Unknown"));
 }
 
+static void
+print_tag (const GstTagList *list, const gchar *tag, gpointer unused)
+{
+  gint i, count;
+
+  count = gst_tag_list_get_tag_size (list, tag);
+
+  for (i = 0; i < count; i++) {
+    gchar *str;
+
+    if (gst_tag_get_type (tag) == G_TYPE_STRING) {
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &str));
+    } else {
+      str = g_strdup_value_contents (
+              gst_tag_list_get_value_index (list, tag, i));
+    }
+
+   if (i == 0) {
+      g_print ("%15s: %s\n", gst_tag_get_nick (tag), str);
+    } else {
+      g_print ("               : %s\n", str);
+    }
+
+    g_free (str);
+  }
+}
+
 /* extract the useful metadata to an allocated string
    returns Unknown if no useful metadata could be found */
 static gchar *
 audio_view_media_get_metadata (GstMediaInfoStream *info)
 {
 	gchar *metadata = NULL;
-	GList *artist = NULL;
-	GList *title = NULL;
-	gchar *artists;
-	gchar *titles;
+	gchar *artist;
+	gchar *title;
 
-	GList *p;
+	GList *p = NULL;
 	GstMediaInfoTrack *track;
 
 	NM_DEBUG("getting metadata\n");
-	if (info->length_tracks < 1) return NULL;
-	if (info->tracks == NULL) return NULL;
+	if (info->length_tracks < 1)
+        {
+		NM_DEBUG("number of tracks < 1, returning\n");
+		return NULL;
+	}
+	if (info->tracks == NULL)
+        {
+		NM_DEBUG("no tracks, returning\n");
+		return NULL;
+	}
 
 	track = info->tracks->data;
-	if (track == NULL) return NULL;
-	if (track->metadata == NULL) return NULL;
-#if 0
-	if (track->metadata->properties == NULL) return NULL;
+	if (track == NULL)
+        {
+		NM_DEBUG("first track NULL, returning\n");
+		return NULL;
+	}
+	if (track->metadata == NULL)
+        {
+		NM_DEBUG("first track's metadata NULL, returning\n");
+		return NULL;
+	}
+	gst_tag_list_get_string (track->metadata, "artist", &artist);
+	gst_tag_list_get_string (track->metadata, "title", &title);
+	NM_DEBUG("Got artists %s, title %s\n", artist, title);
 
-	p = track->metadata->properties->properties;
-#endif
-
-        if (!p) /* no metadata */
+	if (!artist && !title)
                 return g_strdup (_("Unknown"));
+	if (artist && title)
+		return g_strdup_printf ("%s%s\n%s%s",
+			_("Artist: "), artist,
+			_("Title: "), title);
+	if (artist)
+		return g_strdup_printf ("%s%s", _("Artist: "), artist);
+	return g_strdup_printf ("%s%s", _("Title: "), title);
 
-	/* construct GLists of GstPropsEntries to artist and title */
 #if 0
 FIXME: GstPropsEntry is gone, get the right way of tagging extraction done here
 	while (p)
@@ -453,6 +516,7 @@ audio_view_load_location (AudioView *view, const char *location)
   g_free (view->location);
   view->location = g_strdup (location);
   view->selection = NULL;
+  NM_DEBUG("loading location %s and updating view\n", location);
 
   audio_view_update (view);
 }
@@ -559,29 +623,13 @@ audio_view_set_playing (AudioView *view, GstMediaInfoStream *info,
 }
 
 /* ui callbacks */
-void
-audio_view_error_handler (const gchar *message, gpointer data)
+static void
+audio_view_error_handler (const gchar *message, gpointer *data)
 {
-	GtkWindow *parent = GTK_WINDOW (data);
-	GtkWidget *dialog;
+  AudioView *view;
+  view = (AudioView *) data;
 
-	if (parent == NULL || !GTK_IS_WINDOW (parent))
-	{
-		/* no parent window, so not sure if ui windows will be shown */
-		g_warning (message);
-		/* save it for possible later display */
-		/* FIXME: remove this, we don't get at the view anyway */
-		/*
-		if (view->error_message) g_free (view->error_message);
-		view->error_message = g_strdup (message);
-		*/
-	}
-	dialog = gtk_message_dialog_new (parent, GTK_DIALOG_DESTROY_WITH_PARENT,
-			                 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-					 message);
-	/* destroy it if the dialog is run */
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_NONE)
-		gtk_widget_destroy (dialog);
+  show_error_dialog (view, message);
 }
 
 /* FIXME */
@@ -603,6 +651,11 @@ row_activated_callback (GtkTreeView *tree_view, GtkTreePath *path,
 	gtk_tree_model_get (model, &iter,
 			    AUDIO_INFO_COLUMN, &info,
 			    -1);
+	if (!info)
+	{
+		show_error_dialog (view, "No information on file.");
+		return;
+	}
 	NM_DEBUG("callback: get info %p (%s) playing (iterp %)\n",
 		 info, info->path, &iter);
 	audio_view_set_playing (view, info, NULL);
@@ -635,7 +688,6 @@ prev_activate (GtkButton *prev_button, AudioView *view)
 		gtk_tree_model_get (model, &iter,
 				    AUDIO_INFO_COLUMN, &info,
 				    -1);
-	
 		audio_view_set_playing (view, info, path);
 	}
 	else
@@ -818,6 +870,7 @@ free_iter_list (GList *list)
 	g_list_free (list);
 }
 
+/* FIXME: split this up for better legibility */
 static gboolean
 audio_view_get_media_info_idler (AudioView *view)
 {
@@ -825,6 +878,7 @@ audio_view_get_media_info_idler (AudioView *view)
 	GtkTreeIter iter;
 	GstMediaInfoStream *info = NULL;
 	gchar *metadata;
+	GError *error = NULL;
 
 	if (view->audio_list == NULL)
 	{
@@ -859,10 +913,10 @@ audio_view_get_media_info_idler (AudioView *view)
 			NM_DEBUG("getting media-info for path uri %s\n",
 			         path_uri);
                         if (view->media_info == NULL)
-                        	return FALSE;
+				return FALSE;
 			gst_media_info_read_with_idler (view->media_info,
 							path_uri,
-							GST_MEDIA_INFO_ALL);
+							GST_MEDIA_INFO_ALL, &error);
 			view->stream = NULL;
 			view->state = AUDIO_VIEW_INFO_STATE_IDLER;
 			return TRUE;
@@ -878,11 +932,17 @@ audio_view_get_media_info_idler (AudioView *view)
 			if ((view->stream) == NULL)
 			{
 				if (!gst_media_info_read_idler (view->media_info,
-								&(view->stream)))
+								&(view->stream), &error))
 				{
-						g_print ("WARNING: error on media info idler !");
+						if (error)
+						{
+							show_error_dialog (view, error->message);
+							g_error_free (error);
+						}
+
 						/* FIXME: better cleanup here */
-						audio_view_media_info_next (view);
+						//audio_view_media_info_next (view);
+						return FALSE;
 				}
 				return TRUE;
 			}
@@ -958,6 +1018,7 @@ audio_view_update (AudioView *view)
 	char *path;
         gchar *message;
 	GtkTreeIter iter;
+	GError *error = NULL;
 
 	int file_index;
 	int image_count;
@@ -969,8 +1030,7 @@ audio_view_update (AudioView *view)
 	if (result != GNOME_VFS_OK) {
 		path = (char *) gnome_vfs_get_local_path_from_uri (view->location);
 		message = g_strdup_printf (_("Sorry, but there was an error reading %s."), path);
-		eel_show_error_dialog (message, _("Can't read folder"),
-				       GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view->event_box))));
+		show_error_dialog (view, message);
 		g_free (path);
 		g_free (message);
 
@@ -995,6 +1055,7 @@ audio_view_update (AudioView *view)
                 g_free (escaped_name);
 
 		/* FIXME: check if the mime type is playable */
+                NM_DEBUG("checking if %s is playable\n", current_file_info->name);
 		/* we will get NULL if it's not playable */
 		if (audio_view_is_playable (path_uri))
 		{
@@ -1020,8 +1081,21 @@ audio_view_update (AudioView *view)
 	*/
 	gtk_widget_show (view->widget);
 	/* create a media info object */
-	view->media_info = gst_media_info_new (NULL);
-	NM_DEBUG("adding idler");
+	view->media_info = gst_media_info_new (&error);
+	if (error)
+        {
+		show_error_dialog (view, error->message);
+		g_error_free (error);
+		return;
+        }
+	gst_media_info_set_source (view->media_info, "gnomevfssrc", &error);
+	if (error)
+        {
+		show_error_dialog (view, error->message);
+		g_error_free (error);
+		return;
+        }
+	NM_DEBUG("adding idler\n");
 	view->media_info_idler_id = g_idle_add ((GSourceFunc) audio_view_get_media_info_idler, view);
 }
 
@@ -1234,7 +1308,7 @@ audio_view_new ()
 	if (error)
 	{
 		/* no window yet, so no parent */
-		audio_view_error_handler (error->message, NULL);
+		show_error_dialog (NULL, error->message);
 		g_error_free (error);
 	}
 	if (!IS_AUDIO_PLAY (view->audio_play))
@@ -1246,7 +1320,7 @@ audio_view_new ()
 	/* install the error handler */
 	audio_play_set_error_handler (view->audio_play,
 			              (AudioPlayErrorHandler) audio_view_error_handler,
-				      NULL);
+				      view);
 	audio_sink = gst_gconf_get_default_audio_sink ();
 	/* FIXME: do fallback instead of assert */
 	g_assert (GST_IS_ELEMENT (audio_sink));
